package com.jianxun.gaudi.utils;

import com.drew.imaging.ImageMetadataReader;
import com.drew.imaging.ImageProcessingException;
import com.drew.metadata.Metadata;
import com.drew.metadata.exif.ExifIFD0Directory;
import net.coobird.thumbnailator.Thumbnails;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

public class ImgUtils {

    private static Logger LOG = LoggerFactory.getLogger(ImgUtils.class);

    public static void compressPic(File compressPic){

        LOG.info("===== 上传图片进行压缩 =====");
        int angle = getPicRotateAngle(compressPic);
        LOG.info("===== 上传图片需要旋转角度为:"+angle+" =====");

        try {
            BufferedImage sourceImg = ImageIO.read(compressPic);
            if (sourceImg == null){//不是图片
                return;
            }
            Thumbnails.of(sourceImg)
                    .size(1080,1920)
                    .outputQuality(0.8f)//质量压缩到原来的0.5
                    .rotate(angle)
                    .toFile(compressPic);
        }catch (Exception e){
            LOG.info("图片压缩异常 FileUploadUtils compressPic Error ==========>" + e.getMessage());
        }

    }

    public static int getPicRotateAngle(File compressPic){
        Metadata metadata = null;
        int angle = 0;
        try {
            metadata = ImageMetadataReader.readMetadata(compressPic);
            ExifIFD0Directory directoryOfType = metadata.getFirstDirectoryOfType(ExifIFD0Directory.class);
            if (directoryOfType != null){
                int anInt = directoryOfType.getInt(ExifIFD0Directory.TAG_ORIENTATION);

                switch (anInt){
                    case 1: angle = 0;
                        break;
                    case 3: angle = 180;
                        break;
                    case 6: angle = 90;
                        break;
                    case 8: angle = -90;
                        break;
                    default:
                        angle = 0;
                }
            }
        } catch (ImageProcessingException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }catch (Exception e){
            e.printStackTrace();
        }

        return angle;
    }
}
