package com.jianxun.gaudi.utils;

import java.util.Date;

import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.common.vo.TokenVO;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

public class JwtUtil {
	final static String base64EncodedSecretKey = "cY-!I8aQY0(;Pz/@Slove#3";//私钥

	/**
	 * 
	 * @param userName
	 * @param expirateTime (seconds)
	 * @return
	 */
    public static TokenVO createToken(String userName,int expirateTime) {
    	TokenVO vo = new TokenVO();
    	vo.setSubject(userName);
    	vo.setCreateTime(new Date());
    	vo.setExpireTime(getDeltaTime(vo.getCreateTime(),expirateTime));
    	
    	String token = Jwts.builder()
                .setSubject(userName)
                .setIssuedAt(vo.getCreateTime())
                .setExpiration(vo.getExpireTime())
                .signWith(SignatureAlgorithm.HS256, base64EncodedSecretKey)
                .compact();
    	
    	vo.setToken(token);
        
    	return vo;
    }

    /**
     * @Date:17-12-12 下午6:21
     * @Author:root
     * @Desc:检查token,只要不正确就会抛出异常
     **/
    public static String checkToken(String token) throws MyException {
    	String userId = null;
        try {
            final Claims claims = 
            		Jwts.parser().setSigningKey(base64EncodedSecretKey).parseClaimsJws(token).getBody();
            
            userId = claims.getSubject();
        } catch (ExpiredJwtException e1) {
            throw new MyException(ExceptionMsg.TOKEN_EXPIRED.getCode(),ExceptionMsg.TOKEN_EXPIRED.getMsg());
        } catch (Exception e) {
            throw new MyException(ExceptionMsg.TOKEN_VERIFY_ERROR.getCode(),ExceptionMsg.TOKEN_VERIFY_ERROR.getMsg());
        }
        
        return userId;
    }
    
    private static Date getDeltaTime(Date now,int timeGap) {
		return DateHelper.getDateByOffsetSeconds(now, timeGap);
	}
}
