package com.jianxun.gaudi.common.utils;

import cn.superdesk.libs.common.json.JsonUtils;
import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.reflect.TypeToken;
import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.cp.vo.OrgsVO;
import com.jianxun.gaudi.cp.vo.UserVO;
import com.jianxun.gaudi.qs.service.impl.QuestionnaireSurveyServiceImpl;
import com.jianxun.gaudi.sys.model.SysPushLog;
import com.jianxun.gaudi.sys.service.SysPushLogService;
import com.jianxun.gaudi.utils.HttpClientUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class CheckTokenUtil {

    private static Logger logger = LoggerFactory.getLogger(CheckTokenUtil.class);
    private static SysPushLogService sysPushLogService;

    public static UserVO getUserDataByTokenV2(String token, Integer platfromType) {
        String url =userUrls[0];
        logger.info("getUserDataByTokenV2-url1:{};token:{};platfromType:{}", url,token,platfromType);
        if (userUrls.length > platfromType) {
            url = userUrls[platfromType];
        }
        logger.info("getUserDataByTokenV2-url2:{};token:{};platfromType:{}", url,token,platfromType);
        return getUserDataByToken(url,token);
    }

    @Autowired
    public void setSysPushLogService(SysPushLogService sysPushLogService) {
        CheckTokenUtil.sysPushLogService = sysPushLogService;
    }

    private static String userUrl;

    private static String orgsUrl;

    private static String replyTkUrl;

    private static String replyOrgUrl;

    private static String pushUrl;

    private static String [] userUrls;

    @Value("${gaudi.check.url.users}")
    public void setUserUrls(String userUrl){
        this.userUrls = StringUtils.splitByWholeSeparator(userUrl, ";");
    }


    @Value("${gaudi.check.url.user}")
    public void setUserUrl(String userUrl) {
        CheckTokenUtil.userUrl = userUrl;
    }

    @Value("${gaudi.check.url.orgs}")
    public void setOrgsUrl(String orgsUrl) {
        CheckTokenUtil.orgsUrl = orgsUrl;
    }

    @Value("${gaudi.check.url.reply.tk}")
    public void setReplyTkUrl(String replyTkUrl) {
        CheckTokenUtil.replyTkUrl = replyTkUrl;
    }

    @Value("${gaudi.check.url.reply.org}")
    public void setReplyOrgUrl(String replyOrgUrl) {
        CheckTokenUtil.replyOrgUrl = replyOrgUrl;
    }

    @Value("${gaudi.check.url.push}")
    public void setPushUrl(String pushUrl) {
        CheckTokenUtil.pushUrl = pushUrl;
    }

    /**
     *根据token获得用户数据
     * @param token
     * @return
     */
    public static UserVO getUserDataByToken(String token) {
        return getUserDataByToken(userUrl,token);
    }

    public static UserVO getUserDataByToken(String url,String token) {
        JsonObject returnData = postByToken(url, token);
        System.err.println(returnData);
        String code = returnData.get("code").getAsString();
        if(code.equals("200")){
            JsonElement data = returnData.get("data").getAsJsonObject();
            Gson gson = new Gson();
            UserVO userVO = gson.fromJson(data, UserVO.class);
            return userVO;
        }else{
            throw new MyException(ExceptionMsg.TOKEN_VERIFY_ERROR);
        }
    }

    /**
     * 根据token获得项目数据
     * @param token
     * @return
     */
    public static List<OrgsVO> getOrgsByToken(String token) {
        JsonObject returnData = postByToken(orgsUrl, token);
        System.err.println(returnData);
        String code = returnData.get("code").getAsString();
        if(code.equals("200")){
            JsonElement data = returnData.get("data").getAsJsonArray();
            Gson gson = new Gson();
            List<OrgsVO> userVOList = gson.fromJson(data, new TypeToken<List<OrgsVO>>(){}.getType());
            return userVOList;
        }else{
            throw new MyException(ExceptionMsg.TOKEN_VERIFY_ERROR);
        }
    }

    /**
     * 根据token获得某人是否有投诉表扬回复权限
     * @param token
     * @return
     */
    public static boolean getUserIsHaveTsbyReplyByToken(String token) {
        JsonObject returnData = postByToken(replyTkUrl, token);
        System.err.println(returnData);
        String code = returnData.get("code").getAsString();
        if(code.equals("200")){
            String haveTsbyReplyPermission = returnData.get("data").getAsJsonObject().get("haveTsbyReplyPermission").getAsString();
            System.err.println(haveTsbyReplyPermission);
            if("1".equals(haveTsbyReplyPermission)){
                return true;
            }
            return false;
        }else{
            throw new MyException(ExceptionMsg.TOKEN_VERIFY_ERROR);
        }
    }

    /**
     * 根据项目Id获得有投诉表扬回复权限的人
     * @param token
     * @param orgId
     * @param commentId
     * @return
     */
    public static List<String> getUsersHaveTsbyReplyByOrgId(String token, String orgId, String commentId) {
        Map<String, String> map = new HashMap<>();
        map.put("orgId", orgId);
        JsonObject returnData = postByTokenAndMap(replyOrgUrl, token, map);
        System.err.println(returnData);
        String code = returnData.get("code").getAsString();
        String msg = returnData.get("msg").getAsString();
        if(code.equals("200")){
            JsonObject data = returnData.get("data").getAsJsonObject();
            JsonElement users = data.get("users").getAsJsonArray();
            Gson gson = new Gson();
            List<String> userList = gson.fromJson(users, new TypeToken<List<String>>(){}.getType());
            return userList;
        }else{
            SysPushLog sysPushLog = new SysPushLog();
            sysPushLog.setCommentId(commentId);
            sysPushLog.setStatus(0);
            sysPushLog.setContent(msg);
            sysPushLogService.createSysPushLog(sysPushLog);
            return null;
        }
    }

    /**
     * 投诉表扬推送app消息接口
     * @param users
     * @param commentId
     * @param title
     */
    public static void sendPushMessage(List<String> users, String commentId, String title) {
        Map<String, String> map = new HashMap<>();
        map.put("id", commentId);
        map.put("code", "4101");
        map.put("title", "你有新的待回复投诉表扬任务，请及时处理！");
        map.put("content", title);
        String s = "";
        for(String str: users){
            s += str + ";";
        }
        s = s.substring(0, s.length() - 1);
        map.put("users", s);
        JsonObject returnData = postByMap(pushUrl, map);
        System.err.println(returnData);
        String code = returnData.get("code").getAsString();
        String msg = returnData.get("msg").getAsString();
        Date now = new Date();
        SysPushLog sysPushLog = new SysPushLog();
        sysPushLog.setCommentId(commentId);
        if(code.equals("200")){
            sysPushLog.setStatus(1);
            sysPushLog.setContent("成功");
            sysPushLog.setCreateTime(now);
            sysPushLogService.createSysPushLog(sysPushLog);
        }else{
            sysPushLog.setStatus(0);
            sysPushLog.setContent(msg);
            sysPushLogService.createSysPushLog(sysPushLog);
        }
    }

    private static JsonObject postByToken(String url, String token) {
        String res = HttpClientUtils.doPost(url, "token", token);
        return new JsonParser().parse(res).getAsJsonObject();
    }

    private static JsonObject postByTokenAndMap(String url, String token, Map<String, String> map) {
        String res = HttpClientUtils.doPost(url, map,"token", token);
        return new JsonParser().parse(res).getAsJsonObject();
    }

    private static JsonObject postByMap(String url, Map<String, String> map) {
        String res = HttpClientUtils.doPost(url, map);
        return new JsonParser().parse(res).getAsJsonObject();
    }
}
