package com.jianxun.gaudi.common.utils;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.jianxun.gaudi.common.consts.SystemConsts;
import com.jianxun.gaudi.common.enums.RedisKeyPrefix;
import com.jianxun.gaudi.common.vo.RedisVO;
import com.jianxun.gaudi.cp.vo.OrgsVO;
import com.jianxun.gaudi.cp.vo.UserVO;
import com.jianxun.gaudi.sys.service.RedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Component
public class RedisTokenUtil {

    private static RedisService redisService;

    @Autowired
    public void setRedisService(RedisService redisService) {
        RedisTokenUtil.redisService = redisService;
    }

    /**
     * 存储token以及用户资料进redis
     * @param token
     * @return
     */
    public static RedisVO setRedis(String token){
        RedisVO redisVO = new RedisVO();
        UserVO userVO = CheckTokenUtil.getUserDataByToken(token);
        List<OrgsVO> orgsVOList = CheckTokenUtil.getOrgsByToken(token);
        List<String> orgs = orgsVOList.stream().map(p -> p.getId().toString()).distinct().collect(Collectors.toList());
        Map<String,String> mapValue = Maps.newHashMap();
        Gson gson = new Gson();
        mapValue.put("user", gson.toJson(userVO));
        mapValue.put("orgs", gson.toJson(orgs));
        redisService.saveHash(mapValue, RedisKeyPrefix.TOKEN_CACHE, token, SystemConsts.RedisExpireTime.REFRESH_TOKEN_EXPIRE_TIME);
        redisVO.setUserVO(userVO);
        redisVO.setOrgs(orgs);
        return redisVO;
    }

    /**
     * 根据token获取用户资料，如果不存在redis，调远端接口获取并保存redis
     * @param token
     * @return
     */
    public static RedisVO getInfoByRedis(String token){
        RedisVO redisVO = new RedisVO();
        if(redisService.exists(redisService.getCacheKey(RedisKeyPrefix.TOKEN_CACHE, token))){
            String userJson = redisService.hget(redisService.getCacheKey(RedisKeyPrefix.TOKEN_CACHE, token), "user");
            String orgsJson = redisService.hget(redisService.getCacheKey(RedisKeyPrefix.TOKEN_CACHE, token), "orgs");
            Gson gson = new Gson();
            UserVO userVO = gson.fromJson(userJson, UserVO.class);
            List<String> orgs = gson.fromJson(orgsJson, new TypeToken<List<String>>(){}.getType());
            redisVO.setUserVO(userVO);
            redisVO.setOrgs(orgs);
        }else{
            redisVO = RedisTokenUtil.setRedis(token);
        }
        return redisVO;
    }



}
