package com.jianxun.gaudi.cp.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.common.utils.CheckTokenUtil;
import com.jianxun.gaudi.common.utils.RedisTokenUtil;
import com.jianxun.gaudi.common.vo.RedisVO;
import com.jianxun.gaudi.common.vo.StatisticParam;
import com.jianxun.gaudi.common.vo.StatisticVo;
import com.jianxun.gaudi.cp.dao.CpCommentDAO;
import com.jianxun.gaudi.cp.dao.CpCommentFileDAO;
import com.jianxun.gaudi.cp.dao.CpReplyDAO;
import com.jianxun.gaudi.cp.model.*;
import com.jianxun.gaudi.cp.service.CpCommentService;
import com.jianxun.gaudi.cp.vo.CpCommentAppVO;
import com.jianxun.gaudi.cp.vo.CpCommentFromVO;
import com.jianxun.gaudi.cp.vo.UserVO;
import com.jianxun.gaudi.sys.service.SysNumberAssignService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


@Service
@Transactional
public class CpCommentServiceImpl implements CpCommentService {

	private static  Logger   logger = LoggerFactory.getLogger(CpCommentServiceImpl.class);
	
    @Autowired
    CpCommentFileDAO cpCommentFileDAO;

    @Autowired
    CpCommentDAO cpCommentDAO;

    @Autowired
    CpReplyDAO cpReplyDAO;

    @Autowired
    SysNumberAssignService service;

    @Override
    public void createCpComment(String token, CpComment cpComment) {

        /**通过token获取用户**/
        RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
        UserVO userVO = redisVO.getUserVO();
        /**
         * 1.表扬 2.投诉 3.建议
         */
        String[] type =  new String[]{"", "BY", "TS", "JY"};
        /**
         * 不传入类型
         */
        if(cpComment.getType() == null){
            throw new MyException(ExceptionMsg.PARAMETER_INCOMPLETE);
        }
        String id = service.assignNumber(type[cpComment.getType()]);
        CpCommentExample example = new CpCommentExample();
        example.createCriteria().andCommentIdEqualTo(id);
        List<CpComment> cpComments = cpCommentDAO.selectByExample(example);
        if (cpComments.isEmpty()){
            Date now = new Date();
            cpComment.setCommentId(id);
            cpComment.setCreateTime(now);
            cpComment.setProjectId(userVO.getOrgId().toString());
            cpComment.setProjectName(userVO.getOrgName());
            cpComment.setEnterpriseId(userVO.getEnterpriseId().toString());
            cpComment.setEnterpriseName(userVO.getEnterpriseName());
            cpComment.setCreateBy(userVO.getUserId().toString());
            cpComment.setCreateName(userVO.getUserName());
            cpComment.setPhone(userVO.getMobile());
            cpComment.setStatus(0);
            cpCommentDAO.insertSelective(cpComment);
        } else {
            throw new MyException(ExceptionMsg.COMMENT_ID_DUPLICATION);
        }

        if(cpComment.getFile() != null && cpComment.getFile().size() > 0){
            List<CpCommentFile> list = Lists.newArrayList();
            for (String file: cpComment.getFile()) {
                CpCommentFile cpCommentFile = new CpCommentFile();
                cpCommentFile.setCommentId(id);
                cpCommentFile.setFile(file);
                list.add(cpCommentFile);
            }
            cpCommentFileDAO.insertBatch(list);
        }
        List<String> users = CheckTokenUtil.getUsersHaveTsbyReplyByOrgId(token, userVO.getOrgId().toString(), id);
        if(users != null && users.size()>=1){
        	try {//推送失败不影响 订单创建
        		 CheckTokenUtil.sendPushMessage(users, id, cpComment.getTitle());
        	}catch (Exception e) {
        		logger.error("createCpComment CheckTokenUtil.sendPushMessage==>",e);
			}
           
        }
    }

    @Override
    public PageInfo<CpComment> getCpComment(String token, CpCommentFromVO cpCommentFromVO) {

        /**通过token获取项目**/
        RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
        List<String> idList = redisVO.getOrgs();

        PageHelper.startPage(cpCommentFromVO.getCurrentPage(), cpCommentFromVO.getPageSize());

        List<CpComment> cpCommentVOList = Lists.newArrayList();
        if(idList.size() > 0) {
            cpCommentVOList = cpCommentDAO.selectCpComment(idList, cpCommentFromVO.getId(), cpCommentFromVO.getCommentId(), cpCommentFromVO.getProjectId(),
                    cpCommentFromVO.getType(), cpCommentFromVO.getCreateStartTime(), cpCommentFromVO.getCreateEndTime(),
                    cpCommentFromVO.getPhone(), cpCommentFromVO.getReplyStartTime(), cpCommentFromVO.getReplyEndTime(),
                    cpCommentFromVO.getStatus(), cpCommentFromVO.getReplyName());
        }

        for(CpComment cpComment1: cpCommentVOList){
            /**获取每个表扬评论图片**/
            CpCommentFileExample example1 = new CpCommentFileExample();
            CpCommentFileExample.Criteria criteria1 = example1.createCriteria();
            criteria1.andCommentIdEqualTo(cpComment1.getCommentId());
            List<CpCommentFile> fileList = cpCommentFileDAO.selectByExample(example1);
            List<String> urlList = fileList.stream().map(p -> p.getFile()).distinct().collect(Collectors.toList());
            cpComment1.setFile(urlList);
            /**获取每个表扬评论回复**/
            CpReplyExample example2 = new CpReplyExample();
            CpReplyExample.Criteria criteria2 = example2.createCriteria();
            criteria2.andCommentIdEqualTo(cpComment1.getCommentId());
            List<CpReply> cpReplies = cpReplyDAO.selectByExample(example2);
            cpComment1.setCpReplys(cpReplies);
        }
        PageInfo<CpComment> pageinfo = new PageInfo<CpComment>(cpCommentVOList);
        return pageinfo;
    }

    @Override
    public PageInfo<CpCommentAppVO> getCpCommentApp(String token, CpCommentFromVO cpCommentFromVO) {

        /**通过token获取项目**/
        RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
        List<String> idList = redisVO.getOrgs();
        PageHelper.startPage(cpCommentFromVO.getCurrentPage(), cpCommentFromVO.getPageSize());
        List<CpCommentAppVO> cpCommentVOList = Lists.newArrayList();
        if(idList.size() > 0){
            cpCommentVOList = cpCommentDAO.selectCpCommentApp(idList, cpCommentFromVO.getProjectId(),
                    cpCommentFromVO.getType());
        }
        return new PageInfo<CpCommentAppVO>(cpCommentVOList);
    }

    @Override
    public CpComment getCpCommentDetailApp(String token, CpCommentFromVO cpCommentFromVO) {

        CpComment cpComment = new CpComment();
        cpComment = cpCommentDAO.selectCpCommentDetailApp(cpCommentFromVO.getCommentId());

        /**获取表扬评论图片**/
        CpCommentFileExample example = new CpCommentFileExample();
        CpCommentFileExample.Criteria criteria = example.createCriteria();
        criteria.andCommentIdEqualTo(cpComment.getCommentId());
        List<CpCommentFile> fileList = cpCommentFileDAO.selectByExample(example);
        List<String> urlList = fileList.stream().map(p -> p.getFile()).distinct().collect(Collectors.toList());
        cpComment.setFile(urlList);

        /**获取表扬评论回复**/
        CpReplyExample example2 = new CpReplyExample();
        CpReplyExample.Criteria criteria2 = example2.createCriteria();
        criteria2.andCommentIdEqualTo(cpComment.getCommentId());
        List<CpReply> cpReplies = cpReplyDAO.selectByExample(example2);
        cpComment.setCpReplys(cpReplies);

        return cpComment;
    }

    @Override
    public PageInfo<CpComment> getCpCommentWX(String token, CpCommentFromVO cpCommentFromVO) {

        PageHelper.startPage(cpCommentFromVO.getCurrentPage(), cpCommentFromVO.getPageSize());

        /**通过token获取用户**/
        RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
        UserVO userVO = redisVO.getUserVO();

        CpCommentExample example = new CpCommentExample();
        CpCommentExample.Criteria criteria = example.createCriteria();

        criteria.andCreateByEqualTo(userVO.getUserId().toString());
        if(userVO.getOrgId() != null){
            criteria.andProjectIdEqualTo(userVO.getOrgId().toString());
        }
        if(cpCommentFromVO.getType() != null && !cpCommentFromVO.getType().equals(0)){
            criteria.andTypeEqualTo(cpCommentFromVO.getType());
        }
        example.setOrderByClause("create_time DESC");
        List<CpComment> cpCommentVOList = cpCommentDAO.selectByExample(example);

        for(CpComment cpComment : cpCommentVOList){
            /**获取表扬评论图片**/
            CpCommentFileExample example1 = new CpCommentFileExample();
            CpCommentFileExample.Criteria criteria1 = example1.createCriteria();
            criteria1.andCommentIdEqualTo(cpComment.getCommentId());
            List<CpCommentFile> fileList = cpCommentFileDAO.selectByExample(example1);
            List<String> urlList = fileList.stream().map(p -> p.getFile()).distinct().collect(Collectors.toList());
            cpComment.setFile(urlList);

            /**获取每个表扬评论回复**/
            CpReplyExample example2 = new CpReplyExample();
            CpReplyExample.Criteria criteria2 = example2.createCriteria();
            criteria2.andCommentIdEqualTo(cpComment.getCommentId());
            List<CpReply> cpReplies = cpReplyDAO.selectByExample(example2);
            cpComment.setCpReplys(cpReplies);
        }
        return new PageInfo<CpComment>(cpCommentVOList);
    }

    @Override
    public Integer getCommentAmount(String token) {
//        if(CheckTokenUtil.getUserIsHaveTsbyReplyByToken(token)){
//            /**通过token获取项目**/
//            RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
//            List<String> idList = redisVO.getOrgs();
//            Integer amount = cpCommentDAO.getCommentAmount(idList);
//            return amount;
//        }else{
//            return 0;
//        }
        /**通过token获取项目**/
        RedisVO redisVO = RedisTokenUtil.getInfoByRedis(token);
        List<String> idList = redisVO.getOrgs();
        Integer amount = cpCommentDAO.getCommentAmount(idList);
        return amount;
    }

    @Override
    public List<StatisticVo> getStatisticalData(StatisticParam param) {
        return cpCommentDAO.getStatisticalData(param);
    }
}
