package com.jianxun.gaudi.qs.service.impl;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

import cn.superdesk.libs.common.json.JsonUtils;
import com.alibaba.dubbo.common.URL;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jianxun.gaudi.common.vo.StatisticParam;
import com.jianxun.gaudi.common.vo.StatisticSumVo;
import com.jianxun.gaudi.common.vo.StatisticVo;
import com.jianxun.gaudi.cp.model.CpComment;
import com.jianxun.gaudi.qs.dao.*;
import com.jianxun.gaudi.qs.model.*;
import com.jianxun.gaudi.qs.param.*;

import com.jianxun.gaudi.qs.vo.*;
import com.jianxun.gaudi.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.google.gson.internal.LinkedTreeMap;
import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.common.utils.CheckTokenUtil;
import com.jianxun.gaudi.cp.vo.UserVO;
import com.jianxun.gaudi.qs.service.QuestionnaireSurveyService;
import org.springframework.util.CollectionUtils;

@Service
public class QuestionnaireSurveyServiceImpl implements QuestionnaireSurveyService {

    private Logger logger = LoggerFactory.getLogger(QuestionnaireSurveyServiceImpl.class);

    @Value("${gaudi.qs.appid}")
    private String appid;
    @Value("${gaudi.qs.appkey}")
    private String appKey;
    @Value("${gaudi.qs.offical.website}")
    private String officalWebsite;
    @Value("${gaudi.qs.url.login}")
    private String loginUrl;
    @Value("${gaudi.qs.url.tofill}")
    private String toFillUrl;

//    @Value("${gaudi.qs.url.tofill11}")
//    private String url;

    @Value("${gaudi.qs.url.filled}")
    private String filledUrl;
    @Value("${gaudi.qs.url.answer}")
    private String answerUrl;
    @Value("${gaudi.qs.url.question.params}")
    private String questionParams;

    @Value("${qs.wx.message.url}")
    private String wxMessageUrl;


    @Value("${qs.user.url}")
    private String userUrl;

    @Autowired
    QsAnswerDAO qsAnswerDAO;

    @Autowired
    QsQuestionnaireReleaseDAO qsQuestionnaireReleaseDAO;

    @Autowired
    QsQuestionnaireUrgeDAO qsQuestionnaireUrgeDAO;

    @Autowired
    QsQuestionnaireSubgroupDAO qsQuestionnaireSubgroupDAO;

    @Autowired
    QsQuestionnaireSettingDAO qsQuestionnaireSettingDAO;
    @Autowired
    QsQuestionnaireUserDAO qsQuestionnaireUserDAO;

    private Long typea;

    public Long getTypea() {
        return typea;
    }

    public void setTypea(Long typea) {
        this.typea = typea;
    }

    private String user;

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    @Override
    public ManagerVO getManager(String token) throws Exception {
        // 通过token获取用户资料

        //UserVO userVO = CheckTokenUtil.getUserDataByToken(token);

        String platfromType = "0"; //默认为0 服务之窗 1为高校
        String newToken = token;
        if (token.contains("-")) {
            String[] tokens = token.split("-");
            newToken = tokens[0];
            platfromType = tokens[1];
        }
        UserVO userVO = CheckTokenUtil.getUserDataByTokenV2(newToken, Integer.parseInt(platfromType));

        this.check(userVO);

        String currentUser = userVO.getWjxAccount();
        String mobile = userVO.getMobile();
        // 获取时间戳
        String ts = Long.toString(System.currentTimeMillis());
        // 获取签名
        String sign = appid + appKey + currentUser + mobile + ts;
        sign = ShaUtil.shaEncode(sign);
        // 替换登录Url的字符串
        String url = loginUrl.replace("{$appid$}", appid)
                .replace("{$username$}", currentUser)
                .replace("{$mobile$}", mobile)
                .replace("{$ts$}", ts)
                .replace("{$sign$}", sign);
        // 返回结果
        ManagerVO managerVO = new ManagerVO();
        managerVO.setUrl(url);
        return managerVO;
    }

    @Override
    public List<QuestionairesVO> lstToFill(String token) throws Exception {
        // 通过token获取用户资料

//		UserVO userVO = CheckTokenUtil.getUserDataByToken(token);
        String platfromType = "0"; //默认为0 服务之窗 1为高校
        String newToken = token;
        if (token.contains("-")) {
            String[] tokens = token.split("-");
            newToken = tokens[0];
            platfromType = tokens[1];
        }
        long beginTime = System.currentTimeMillis();
        logger.info("问卷列表-getManager-lstToFill newToken:{};获取用户信息开始时间:{}",newToken,beginTime);
        UserVO userVO = CheckTokenUtil.getUserDataByTokenV2(newToken, Integer.parseInt(platfromType));
        long endTime = System.currentTimeMillis();
        logger.info("问卷列表-getManager-lstToFill userVO:{};获取用户信息结束时间:{}；耗时:{}",JsonUtils.toJson(userVO),endTime,(beginTime-endTime));

        this.check(userVO);
        String realname = StringUtils.isEmpty(userVO.getUserName()) || userVO.getUserName().length() > 10 ? "" : userVO.getUserName(); // 填写者，真实姓名

        String username = userVO.getWjxAccount();// 发布者
        String joiner = userVO.getUserId().toString();// 填写者
        String dept = "";// 部门
        String extf = platfromType;// 附加字段，不能超过1000个字符，作为填写者信息的补充
        // 获取时间戳
        String ts = Long.toString(System.currentTimeMillis());

        // 获取签名
        String sign = appid + appKey + username + joiner + realname + dept + extf + ts;
        sign = ShaUtil.shaEncode(sign);
        // 构建接口地址
        String url = toFillUrl.replace("{$appid$}", appid)
                .replace("{$username$}", username)
                .replace("{$joiner$}", joiner)
                .replace("{$realname$}", realname)
                .replace("{$dept$}", dept)
                .replace("{$extf$}", extf)
                .replace("{$ts$}", ts)
                .replace("{$sign$}", sign);
        long bTime = System.currentTimeMillis();
        logger.info("问卷列表-getManager-lstToFill url:{};请求问卷星接口开始时间:{}",url,bTime);
        // 调用接口并获取结果
        String result = HttpClientUtils.doGet(url);
        long eTime = System.currentTimeMillis();
        logger.info("问卷列表-getManager-lstToFill result:{};请求问卷星接口结束时间:{};所用时间差：{}",result,eTime,(eTime-bTime));
        // 问题传递参数
        String questionParamsString = questionParams.replace("{$sojumpparm$}", token);
        logger.info("lstToFill questionParams===>" + questionParams);
        logger.info("lstToFill questionParamsString===>" + questionParamsString);
        logger.info("lstToFill token===>" + token);
//        List<QuestionairesVO> lstQuestionairesVO = JsonUtil.<QuestionairesVO>fromJsonToList(result);
        setTypea(typea);
        setUser(joiner);
        List<QuestionairesVO> lstQuestionairesVO = JSON.parseArray(result, QuestionairesVO.class);

        List<QuestionairesVO> vo = new ArrayList<>();
        Long typea = qsQuestionnaireSettingDAO.qsQuestionnaireSetting(userVO.getOrgId());
        logger.info("lstToFill typea===>" + typea);
        List<QsQuestionnaireRelease> list = qsQuestionnaireReleaseDAO.questionnaireScreening(userVO.getUserId(), userVO.getOrgId());
        logger.info("lstToFill list===>" + list.size());
        if (typea != null && typea==1 && list.size() != 0 ) {
            list.forEach(item -> {
                for (String shu : item.getActivity().split(",")) {
                    for (int i = 0; i < lstQuestionairesVO.size(); i++) {
                        if (lstQuestionairesVO.get(i).getQid().equals(shu)) {
                            if (vo.stream().filter(w -> String.valueOf(w.getQid()).equals(shu)).findAny().isPresent()) {
                                //其他操作
                            } else {
                                vo.add(lstQuestionairesVO.get(i));
                            }
                        }
                    }
                }
            });
        } else if(typea == null ||  typea==0){
            vo.addAll(lstQuestionairesVO);
        }

        for (QuestionairesVO questionairesVO : vo) {
            questionairesVO.setLink(officalWebsite + questionairesVO.getLink() + questionParamsString);
        }
        return vo;
    }


    @Override
    public List<QuestionairesVO> getQuestionnaire(QsUserParam param) throws Exception {
        // 通过token获取用户资料
        String realname = "";
        String username = param.getUsername() == null ? "" : param.getUsername();   //hkdspro¤362581 ;// 发布者
        String joiner = "0";  //"140149";// 填写者
        String dept = "";// 部门
        String extf = param.getExtf();// 附加字段，不能超过1000个字符，作为填写者信息的补充
        // 获取时间戳
        String ts = Long.toString(System.currentTimeMillis());
//
//        String sign1=appid + appKey + username + ts;
//        sign1 = ShaUtil.shaEncode(sign1);
//        String url1=url.replace("{$appid$}", appid)
//                .replace("{$username$}", username)
//                .replace("{$ts$}", ts)
//                .replace("{$folder$}", "")
//                .replace("{$sign$}", sign1);
//        String result1111 = HttpClientUtils.doGet(url1);
//        List<QuestionairesVO> lstQuestionairesVO = JSON.parseArray(result1111, QuestionairesVO.class);


        // 获取签名
        String sign = appid + appKey + username + joiner + realname + dept + extf + ts;
        sign = ShaUtil.shaEncode(sign);
        // 构建接口地址
        String url = toFillUrl.replace("{$appid$}", appid)
                .replace("{$username$}", username)
                .replace("{$joiner$}", joiner)
                .replace("{$realname$}", realname)
                .replace("{$dept$}", dept)
                .replace("{$extf$}", extf)
                .replace("{$ts$}", ts)
                .replace("{$sign$}", sign);
        // 调用接口并获取结果
        String result = HttpClientUtils.doGet(url);
        // 问题传递参数


//        List<QuestionairesVO> lstQuestionairesVO = JsonUtil.<QuestionairesVO>fromJsonToList(result);

        List<QuestionairesVO> lstQuestionairesVO = JSON.parseArray(result, QuestionairesVO.class);

        if(CollectionUtils.isEmpty(lstQuestionairesVO)){
            return lstQuestionairesVO;
        }
        // Json转列表

        for (QuestionairesVO questionairesVO : lstQuestionairesVO) {
            if(null == questionairesVO){
                continue;
            }

            questionairesVO.setLink(officalWebsite + questionairesVO.getLink() );
        }

        return lstQuestionairesVO;
    }

    @Override
    public List<QuestionairesVO> lstFilled(String token) throws Exception {
        // 通过token获取用户资料
//		UserVO userVO = CheckTokenUtil.getUserDataByToken(token);
        String platfromType = "0"; //默认为0 服务之窗 1为高校
        String newToken = token;
        if (token.contains("-")) {
            String[] tokens = token.split("-");
            newToken = tokens[0];
            platfromType = tokens[1];
        }
        UserVO userVO = CheckTokenUtil.getUserDataByTokenV2(newToken, Integer.parseInt(platfromType));

        this.check(userVO);

        String realname = StringUtils.isEmpty(userVO.getUserName()) || userVO.getUserName().length() > 10 ? "" : userVO.getUserName(); // 填写者，真实姓名

        String username = userVO.getWjxAccount();// 发布者
        String joiner = userVO.getUserId().toString();// 填写者
        String dept = "";// 部门
        String extf = platfromType;// 附加字段，不能超过1000个字符，作为填写者信息的补充
        // 获取时间戳
        String ts = Long.toString(System.currentTimeMillis());

        // 获取签名
        String sign = appid + appKey + username + joiner + realname + dept + extf + ts;
        sign = ShaUtil.shaEncode(sign);
        // 构建接口地址
        String url = filledUrl.replace("{$appid$}", appid)
                .replace("{$username$}", username)
                .replace("{$joiner$}", joiner)
                .replace("{$realname$}", realname)
                .replace("{$dept$}", dept)
                .replace("{$extf$}", extf)
                .replace("{$ts$}", ts)
                .replace("{$sign$}", sign);
        // 调用接口并获取结果
        String result = HttpClientUtils.doGet(url);
        // Json转列表

        List<QuestionairesVO> lstQuestionairesVO = JsonUtil.<QuestionairesVO>fromJsonToList(result);
        for (Object questionairesVO : lstQuestionairesVO) {
            LinkedTreeMap<String, String> ltQuestionairesVO = (LinkedTreeMap) questionairesVO;
            String qid = ltQuestionairesVO.get("qid");
            String joindate = ltQuestionairesVO.get("joindate");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
            Date dJoindate = sdf.parse(joindate);

            QsAnswerExample qsAnswerExample = new QsAnswerExample();
            qsAnswerExample.or().andActivityEqualTo(qid)
                    .andParterJoinerEqualTo(joiner)
                    .andSubmitTimeEqualTo(dJoindate);
            List<QsAnswer> lstAnswers = qsAnswerDAO.selectByExample(qsAnswerExample);

            if (lstAnswers != null && lstAnswers.size() > 0) {
                QsAnswer qsAnswer = lstAnswers.get(0);

                String joinid = qsAnswer.getJoinId();

                // 获取签名
//				sign = appid + appKey + username + joiner + realname + dept + extf + ts;
                sign = appid + appKey + username + joiner + qid + joinid + realname + dept + extf + ts;
                sign = ShaUtil.shaEncode(sign);
                String urlString = answerUrl.replace("{$activity$}", qid)
                        .replace("{$joinid$}", joinid)
                        .replace("{$username$}", username)
                        .replace("{$joiner$}", joiner)
                        .replace("{$realname$}", realname)
                        .replace("{$dept$}", dept)
                        .replace("{$extf$}", extf)
                        .replace("{$appid$}", appid)
                        .replace("{$ts$}", ts)
                        .replace("{$sign$}", sign);

                ltQuestionairesVO.put("link", urlString);
            } else {
                ltQuestionairesVO.put("link", "");
            }
            ltQuestionairesVO.put("joiner", joiner);
        }

        return lstQuestionairesVO;
    }

    @Override
    public void callBack(QsAnswer qsAnswer) throws Exception {
        logger.info("QuestionnaireSurveyServiceImpl getTypea()===>" + JsonUtils.toJson(getTypea()));
        logger.info("QuestionnaireSurveyServiceImpl getUser()===>" + JsonUtils.toJson(getUser()));
        qsAnswerDAO.insertSelective(qsAnswer);
        logger.info("QuestionnaireSurveyServiceImpl callBack===>" + JsonUtils.toJson(qsAnswer));
        if(getTypea()!=null && getTypea().toString().equals("1")){
            QuestionnaireReminderVO questionnaireReminderVO  =   qsQuestionnaireReleaseDAO.questionnaireId(qsAnswer.getActivity(),getUser());
            logger.info("QuestionnaireSurveyServiceImpl questionnaireReminderVO===>" + JsonUtils.toJson(questionnaireReminderVO));
            qsQuestionnaireUserDAO.updateSelective(qsAnswer.getParterJoiner()==null?getUser():qsAnswer.getParterJoiner(),questionnaireReminderVO.getId());
        }

    }

    /**
     * 用户信息检查
     *
     * @param userVO
     * @throws Exception
     */
    private void check(UserVO userVO) throws Exception {
        if (StringUtils.isBlank(userVO.getWjxAccount()))
            throw new MyException(ExceptionMsg.CYCLE_EMPTY_WJXACCOUNT);
        if (userVO.getUserId() == null)
            throw new MyException(ExceptionMsg.CYCLE_EMPTY_USERID);
    }

    @Override
    public QsAnswerVO getAnswerUrl(String token, QuestionairesVO questionairesVO) throws Exception {
        // 通过token获取用户资料
//		UserVO userVO = CheckTokenUtil.getUserDataByToken(token);
        String platfromType = "0"; //默认为0 服务之窗 1为高校
        String newToken = token;
        if (token.contains("-")) {
            String[] tokens = token.split("-");
            newToken = tokens[0];
            platfromType = tokens[1];
        }
        UserVO userVO = CheckTokenUtil.getUserDataByTokenV2(newToken, Integer.parseInt(platfromType));

        String realname = StringUtils.isEmpty(userVO.getUserName()) || userVO.getUserName().length() > 10 ? "" : userVO.getUserName(); // 填写者，真实姓名

        this.check(userVO);
        String username = userVO.getWjxAccount();// 发布者
        String joiner = userVO.getUserId().toString();// 填写者
        String dept = "";// 部门
        String extf = platfromType;// 附加字段，不能超过1000个字符，作为填写者信息的补充
        // 获取时间戳
        String ts = Long.toString(System.currentTimeMillis());
        // 参与日期
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        Date dJoindate = sdf.parse(questionairesVO.getJoindate());
        // 问卷ID
        String qid = questionairesVO.getQid();
        // 返回数据
        QsAnswerVO qsAnswerVO = new QsAnswerVO();
        // 取问卷答案信息
        QsAnswerExample qsAnswerExample = new QsAnswerExample();
        qsAnswerExample.or().andActivityEqualTo(qid)
                .andParterJoinerEqualTo(joiner)
                .andSubmitTimeEqualTo(dJoindate);
        List<QsAnswer> lstAnswers = qsAnswerDAO.selectByExample(qsAnswerExample);
        if (lstAnswers != null && lstAnswers.size() > 0) {
            QsAnswer qsAnswer = lstAnswers.get(0);
            // 用户参与ID
            String joinid = qsAnswer.getJoinId();
            // 获取签名
            String sign = appid + appKey + username + joiner + qid + joinid + realname + dept + extf + ts;
            sign = ShaUtil.shaEncode(sign);
            String urlString = answerUrl.replace("{$activity$}", qid)
                    .replace("{$joinid$}", joinid)
                    .replace("{$username$}", username)
                    .replace("{$joiner$}", joiner)
                    .replace("{$realname$}", realname)
                    .replace("{$dept$}", dept)
                    .replace("{$extf$}", extf)
                    .replace("{$appid$}", appid)
                    .replace("{$ts$}", ts)
                    .replace("{$sign$}", sign);

            qsAnswerVO.setUrl(urlString);
        } else {
            qsAnswerVO.setUrl("");
        }


        return qsAnswerVO;
    }

    @Override
    public void addQuestionnaire(QsQuestionnaireParam param) throws Exception {
        String id = String.format("%016d", Math.abs(UUID.randomUUID().toString().hashCode()));  //转化为String对象
        //问卷id
        String activity = param.getQuestionnaireMethodParams().stream().map(QuestionnaireMethodParam::getActivity).collect(Collectors.joining(","));
        //问卷名称
        String name = param.getQuestionnaireMethodParams().stream().map(QuestionnaireMethodParam::getName).collect(Collectors.joining(","));
        //部门/分组名称
        String deptGroupName = param.getQsQuestionnaireSubgroups().stream().map(QsQuestionnaireSubgroup::getDeptGroupName).collect(Collectors.joining(","));
        QsQuestionnaireRelease qsQuestionnaireRelease = new QsQuestionnaireRelease();
        BeanUtils.copyProperties(param, qsQuestionnaireRelease);
        qsQuestionnaireRelease.setActivity(activity);
        qsQuestionnaireRelease.setName(name);
        qsQuestionnaireRelease.setCreatedat(new Date());
        qsQuestionnaireRelease.setQuestionnaireId(id);
        qsQuestionnaireRelease.setSender(deptGroupName);
        qsQuestionnaireRelease.setUrl(param.getQuestionnaireMethodParams().get(0).getUrl());
        //增加问卷发布管理
        qsQuestionnaireReleaseDAO.insertSelective(qsQuestionnaireRelease);

        Calendar calendar = Calendar.getInstance();
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        if(param.getDeadline()!=null && param.getQsQuestionnaireUrges()!=null && !param.getQsQuestionnaireUrges().isEmpty()) {
            param.getQsQuestionnaireUrges().stream().map(person -> {
                person.setSourceId(id);
                calendar.setTime(param.getDeadline());
                calendar.add(calendar.HOUR_OF_DAY, -person.getDistanceCutoff());
                person.setTime(df.format(calendar.getTime()));
                return person;
            }).collect(Collectors.toList());
            //增加自动催收
            qsQuestionnaireUrgeDAO.insertSelective(param.getQsQuestionnaireUrges());
        }

        param.getQsQuestionnaireSubgroups().stream().map(person -> {
            person.setSourceId(id);
            return person;
        }).collect(Collectors.toList());
        //增加问卷-部门
        qsQuestionnaireSubgroupDAO.insertSelective(param.getQsQuestionnaireSubgroups());
        //增加用户信息
        DeptGroupVO deptGroupVO = new DeptGroupVO();
        deptGroupVO.setOrgId(param.getOrgId());
        DeptGroupTypeVO deptGroupTypeVO = new DeptGroupTypeVO();
        param.getQsQuestionnaireSubgroups().forEach(item -> {
            List<MsgUtil> msgUtil=new ArrayList<>();
            List<DeptGroupTypeVO> list = new ArrayList();
            deptGroupTypeVO.setType(item.getDeptGroupType());
            deptGroupTypeVO.setId(item.getDeptGroupId());
            list.add(deptGroupTypeVO);
            deptGroupVO.setList(list);
            List<DeptUserVO> deptUserVO = getDeptGroupUserListByOrgId(deptGroupVO);
            if(deptUserVO!=null && !deptUserVO.isEmpty()){
                deptUserVO.stream().map(it -> {
                    it.setSourceId(id);
                    it.setDeptGroupId(item.getDeptGroupId());
                    it.setDeptGroupName(item.getDeptGroupName());
                    MsgUtil util=new MsgUtil();
                    util.setOrgId(param.getOrgId());
                    util.setUserId(it.getUserId());
                    util.setUrl(qsQuestionnaireRelease.getUrl());
                    util.setContent("您好,请在规定时间内填写问卷");
                    util.setName(name);
                    msgUtil.add(util);
                    return it;
                }).collect(Collectors.toList());
                qsQuestionnaireUserDAO.insertSelective(deptUserVO);
                //推送
                sendWxCusmonMsg(msgUtil);
            }

        });


    }

    /**
     * 发送客服消息
     *
     * @return
     * @throws Exception
     */
    public void sendWxCusmonMsg(Integer orgId, Integer userId, String url, String content,String name) {
        // 构建接口地址
        try {
            Integer[] userIds = new Integer[]{userId};
            URL.encode(URLEncoder.encode(url, "UTF-8"));
            String wxUrl = wxMessageUrl.replace("{$orgId$}", orgId.toString())
                    .replace("{$userIds$}", StringUtils.join(userIds))
                    .replace("{$content$}", content)
                    .replace("{$url$}", url)
                    .replace("{$name$}", name);

            String result = HttpClientUtils.doPost(wxUrl);
            logger.info("sendWxCusmonMsg result ===>" + result);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }


    }
//    public void sendWxCusmonMsg( List<MsgUtil> list) {
//        // 构建接口地址
//        ExecutorService executorService = Executors.newFixedThreadPool(1);
//        executorService.execute(new Runnable() {
//            public void run() {
//                try {
//                    logger.info("sendWxCusmonMsg=======>>>>>>>>>开始发送消息模板");
//                    String userIds = list.stream().map(MsgUtil::getUserId).collect(Collectors.joining(","));
//                    Map<String,String>map=new HashMap<>();
//                    map.put("orgId",list.get(0).getOrgId().toString());
//                    map.put("userIds",userIds);
//                    map.put("content",list.get(0).getContent());
//                    map.put("url",list.get(0).getUrl());
//                    String params=com.alibaba.fastjson.JSONObject.toJSONString(map);
//                    String result=HttpClientUtils.doPost(wxMessageUrl,map);
//                    System.out.println(result);
//                    logger.info("sendWxCusmonMsg result===>"+  JsonUtils.toJson(result));
//                } catch (Exception e) {
//                    e.printStackTrace();
//                }
//            }
//        });
//        executorService.shutdown();
//    }
    /**
     * 发送客服消息
     *
     * @return
     * @throws Exception
     */
    public void sendWxCusmonMsg(List<MsgUtil> util) {
        // 构建接口地址
        ExecutorService executorService = Executors.newFixedThreadPool(1);
        executorService.execute(new Runnable() {
            public void run() {
                try {
                    for (int i = 0; i < util.size(); i++) {
                        MsgUtil msgUtil=util.get(i);
                        Integer[] userIds = new Integer[]{msgUtil.getUserId()};
                        URL.encode(URLEncoder.encode(msgUtil.getUrl(), "UTF-8"));
                        if(StringUtils.isNotEmpty(msgUtil.getName()) ){
                            msgUtil.setName(msgUtil.getName().replace("[","(").replace("]",")"));
                        }
                        String wxUrl = wxMessageUrl.replace("{$orgId$}", msgUtil.getOrgId().toString())
                                .replace("{$userIds$}", StringUtils.join(userIds))
                                .replace("{$content$}", msgUtil.getContent())
                                .replace("{$url$}",msgUtil.getUrl())
                                .replace("{$name$}", msgUtil.getName());
                        logger.info("QuestionnaireSurveyServiceImpl wxUrl===>" + wxUrl);
                        String result = HttpClientUtils.doPost(wxUrl);
                        logger.info("QuestionnaireSurveyServiceImpl sendWxCusmonMsg===>" + "发送定时模板消息");
                        logger.info("sendWxCusmonMsg result===>"+  JsonUtils.toJson(userIds)+"用户"+ result);
                    }
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
        });
        executorService.shutdown();
    }


    public List<DeptUserVO> getDeptGroupUserListByOrgId(DeptGroupVO vo) {
        List<DeptUserVO> list = new ArrayList<DeptUserVO>();
        try {
            String jsonObject = JSONObject.toJSONString(vo);
            String result = HttpClientUtils.post(userUrl, jsonObject);
            if (StringUtils.isBlank(result)) {
                return list;
            }
            JSONObject jsonObj = JSONObject.parseObject(result);
            if (!jsonObj.containsKey("code") || 200 != jsonObj.getInteger("code") || !jsonObj.containsKey("data")) {
                return list;
            }
            JSONArray jsonArray = jsonObj.getJSONArray("data");
            if (null == jsonArray) {
                return list;
            }
            list = JSONObject.parseArray(jsonArray.toJSONString(), DeptUserVO.class);
            return list;
        } catch (Exception e) {
            e.printStackTrace();
            return list;
        }
    }


    @Override
    public void sendSettings(QsQuestionnaireSettingParam param) throws Exception {
        QsQuestionnaireSetting qsQuestionnaireSetting = new QsQuestionnaireSetting();
        BeanUtils.copyProperties(param, qsQuestionnaireSetting);

        //项目是否存在，存在就修改，不存在就添加
        Long count = qsQuestionnaireSettingDAO.countByExample(qsQuestionnaireSetting.getOrgId());
        if (count != null) {
            qsQuestionnaireSetting.setUpdatedat(new Date());
            qsQuestionnaireSettingDAO.updateSelective(qsQuestionnaireSetting);
        } else {
            qsQuestionnaireSetting.setCreatedat(new Date());
            qsQuestionnaireSettingDAO.insertSelective(qsQuestionnaireSetting);
        }
    }

    @Override
    public void modifyQuestionnaireUser(Integer userId) throws Exception {
//        qsQuestionnaireUserDAO.updateSelective(userId.toString());
    }

    @Override
    public void questionnaireReminder(QsQuestionnaireUrgeParam param) throws Exception {
        logger.info("QuestionnaireSu`rveyServiceImpl questionnaireReminder===>param:{}" + JsonUtils.toJson(param));
        QsQuestionnaireUrge qsQuestionnaireUrge = new QsQuestionnaireUrge();
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        qsQuestionnaireUrge.setTime(df.format(new Date()));
        BeanUtils.copyProperties(param, qsQuestionnaireUrge);
        logger.info("QuestionnaireSu`rveyServiceImpl questionnaireReminder2===>param:{}" + JsonUtils.toJson(param));
        List<QsQuestionnaireUrge> list = new ArrayList<>();
        list.add(qsQuestionnaireUrge);
        qsQuestionnaireUrgeDAO.insertSelective(list);
        logger.info("QuestionnaireSu`rveyServiceImpl questionnaireReminder===>" + "发送催收模板消息:{}",JsonUtils.toJson(param));
        List<QsQuestionnaireUserVO> qsQuestionnaireUserVOS = qsQuestionnaireUserDAO.getUser(param.getSourceId());
        logger.info("QuestionnaireSurveyServiceImpl.qsQuestionnaireUserVOS:{}", JsonUtils.toJson(qsQuestionnaireUserVOS));
        List<MsgUtil>utils=new ArrayList<>();
        qsQuestionnaireUserVOS.forEach(item -> {
            MsgUtil util=new MsgUtil();
            util.setOrgId(param.getOrgId());
            util.setUserId(item.getUserId());
            util.setUrl(param.getUrl());
            util.setContent(param.getSendContent());
            util.setName(param.getName());
            utils.add(util);
        });
        sendWxCusmonMsg(utils);
    }

    @Override
    public QsQuestionnaireSettingVO queryQuestionnaireSettings(Integer orgId) throws Exception {
        QsQuestionnaireSettingVO qsQuestionnaireSettingVO=qsQuestionnaireSettingDAO.selectByPrimary(orgId);
        if(qsQuestionnaireSettingVO==null){
            qsQuestionnaireSettingVO=new QsQuestionnaireSettingVO();
            qsQuestionnaireSettingVO.setOrgId(orgId);
            qsQuestionnaireSettingVO.setType(0);
        }
        return  qsQuestionnaireSettingVO;
    }


    @Override
    public PageInfo<QsQuestionnaireVO> getQuestionnaireInformation(QuestionnaireInquiryParam param) throws Exception {
        PageHelper.startPage(param.getPageNum(), param.getPageSize());
        List<QsQuestionnaireVO> list = qsQuestionnaireReleaseDAO.getQuestionnaireInformation(param);
        return new PageInfo<QsQuestionnaireVO>(list);
    }

    @Override
    public QuestionnaireDetailsVO questionnaireDetails(Integer id) throws Exception {
        QuestionnaireDetailsVO questionnaireDetailsVO = new QuestionnaireDetailsVO();
        List<QsQuestionnaireUserVO> list = qsQuestionnaireUserDAO.selectByPrimaryKeye(id);
        questionnaireDetailsVO.setName(list.get(0).getName());
        questionnaireDetailsVO.setSender(list.get(0).getSender());
        questionnaireDetailsVO.setQsQuestionnaireUserVO(list);
        questionnaireDetailsVO.setQsQuestionnaireUrgeVO(qsQuestionnaireUrgeDAO.selectByExamplek(list.get(0).getQuestionnaireId()));

        return questionnaireDetailsVO;
    }

    @Override
    public List<QsQuestionnaireUserVO> questionnaireProgress(QuestionnaireProgressParam param) throws Exception {
        List<QsQuestionnaireUserVO> questionnaireParticipated = qsQuestionnaireUserDAO.questionnaireProgress(param);
        return questionnaireParticipated;
    }

    @Override
    public List<StatisticVo> getStatisticalData(StatisticParam param) {
        return qsQuestionnaireReleaseDAO.getStatisticalData(param);
    }

    @Override
    public List<StatisticSumVo> getStatisticalSumData() {
        return qsQuestionnaireReleaseDAO.getStatisticalSumData();
    }


    public void delayedOrderTask() {
        logger.info("及时使用提醒开始-------------------");
        List<QuestionnaireReminderVO> list = qsQuestionnaireReleaseDAO.questionnaireCollection();
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        LocalDateTime date = LocalDateTime.now();
        if (list.size()>0){
            list.forEach(item -> {
                if(item.getDistanceCutoff()!=null && item.getDeadline()!=null){
                    LocalDateTime localDateTime = date.plusHours(item.getDistanceCutoff());
                    if(dateTimeFormatter.format(localDateTime).compareTo(item.getDeadline()) == 0 && dateTimeFormatter.format(date).compareTo(item.getDeadline())<0){
                        List<QsQuestionnaireUserVO>vo= qsQuestionnaireUserDAO.questionnaireScreening(item.getQuestionnaireId(),item.getOrgId());
                        vo.forEach(s->{
                            logger.info("QuestionnaireSurveyServiceImpl delayedOrderTask===>" + "发送定时模板消息");
                            sendWxCusmonMsg(item.getOrgId(), s.getUserId(), s.getUrl(), item.getSendContent(),item.getName());
                        });
                    }
                }

            });
            logger.info("及时使用提醒结束1-------------------");
        }else {
            logger.info("及时使用提醒结束2-------------------");
        }


    }


}
