package com.jianxun.gaudi.sys.service;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.jianxun.gaudi.common.enums.RedisKeyPrefix;


public interface RedisService {
	default String getCacheKey(RedisKeyPrefix keyPrefix, String key) {
		return keyPrefix.getKey()+":"+key;
	}
	
	/**
	 * 
	 * 功能说明：将对象保存到redis里（单位秒）
	 * @createTime 2018年4月9日 下午5:25:56
	 * @param value
	 * @param keyPrefix key前缀
	 * @param key key后缀
	 * @param time 时间（单位秒）
	 */
	public void saveObject(String value,RedisKeyPrefix keyPrefix,String key,long time);
	
	
	/**
	 * save hash value in redis (if time <= 0, not expired time)
	 * @param mapValue
	 * @param keyPrefix
	 * @param key
	 * @param time
	 */
	public void saveHash(Map<String,String> mapValue,RedisKeyPrefix keyPrefix,String key,long time);
	
	/**
	 * 将哈希表 key 中的域 field 的值设为 value 。
	 * @param redisKey
	 * @param key
	 * @param value
	 */
	public void hset(String redisKey,String key,String value);
	
	/**
	 * 为哈希表 key 中的域 field 的值加上增量 increment 。
	 * @param redisKey
	 * @param key
	 * @param value
	 * @return
	 */
	public long hincrBy(String redisKey,String key,int value);
	
	/**
	 * 
	 * 功能说明：通过key获取保存到redis里的值
	 * @createTime 2018年4月9日 下午5:26:51
	 * @param keyPrefix key前缀
	 * @param key key后缀
	 * @return
	 */
	public String getKey(RedisKeyPrefix keyPrefix,String key);
	
	/**
	 * 返回redis 哈希表 key 中给定域 field 的值
	 * @param redisKey
	 * @param hashKey
	 * @return
	 */
	public Object hgetKey(String redisKey,String hashKey);
	
	
	/**
	 * 
	 * 功能说明：通过key删除redis里面的值
	 * @createTime 2018年4月9日 下午5:27:14
	 * @param keyPrefix key前缀
	 * @param key key后缀
	 */
	public void deleteKey(RedisKeyPrefix keyPrefix,String key);

	/**
	 * 为给定 key 设置生存时间，当 key 过期时(生存时间为 0 )，它会被自动删除
	 * @param redisKey
	 * @param timeout
	 * @param unit
	 */
	public void expire(String redisKey, long timeout, TimeUnit unit);

	/**
	 * 返回哈希表 key 中给定域 field 的值
	 * @param redisKey
	 * @param hashKey
	 * @return
	 */
	public String hget(String redisKey, String hashKey);

	/**
	 * 将 key 中储存的数字值增一
	 * @param redisKey
	 * @param increment
	 * @return
	 */
	public Long incr(String redisKey,int increment);

	/**
	 * 判断是否存在
	 * @param redisKey
	 * @return
	 */
	public boolean exists(String redisKey);
}
