package com.jianxun.gaudi.sys.service.impl;

import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import com.jianxun.gaudi.common.enums.RedisKeyPrefix;
import com.jianxun.gaudi.sys.service.RedisService;

@Service
public class RedisServiceImpl implements RedisService {

	@Autowired
	private StringRedisTemplate redisTemplate;

	@Override
	public void saveObject(String value, RedisKeyPrefix keyPrefix, String key, long time) {
		String cacheKey = getCacheKey(keyPrefix, key);
		redisTemplate.opsForValue().set(cacheKey, value, time, TimeUnit.SECONDS);
	}
	
	@Override
	public String getKey(RedisKeyPrefix keyPrefix, String key) {
		String cacheKey = getCacheKey(keyPrefix, key);
		return redisTemplate.opsForValue().get(cacheKey);
	}
	
	@Override
	public void deleteKey(RedisKeyPrefix keyPrefix, String key) {
		String cacheKey = getCacheKey(keyPrefix, key);
		redisTemplate.delete(cacheKey);
	}

	@Override
	public void saveHash(Map<String, String> mapValue, RedisKeyPrefix keyPrefix, String key, long time) {
		// TODO Auto-generated method stub
		String cacheKey = getCacheKey(keyPrefix, key);
		
		redisTemplate.opsForHash().putAll(cacheKey, mapValue);
		
		if(time > 0) {
			redisTemplate.expire(cacheKey, time, TimeUnit.SECONDS);
		}
	}

	@Override
	public Object hgetKey(String redisKey, String hashKey) {
		// TODO Auto-generated method stub
		return redisTemplate.opsForHash().get(redisKey, hashKey);
	}

	@Override
	public void hset(String redisKey, String key, String value) {
		// TODO Auto-generated method stub
		redisTemplate.opsForHash().put(redisKey, key, value);
	}

	@Override
	public long hincrBy(String redisKey, String key, int value) {
		// TODO Auto-generated method stub
		return redisTemplate.opsForHash().increment(redisKey, key, value);
	}

	@Override
	public void expire(String redisKey, long timeout,TimeUnit unit) {
		redisTemplate.expire(redisKey, timeout, unit);
	}

	@Override
	public String hget(String redisKey,String hashKey) {
		Object obj = redisTemplate.opsForHash().get(redisKey, hashKey);
		String rsp = obj == null ? "" : obj.toString();
		return rsp;
	}

	@Override
	public Long incr(String redisKey,int increment) {
		return redisTemplate.opsForValue().increment(redisKey, increment);
	}

	@Override
	public boolean exists(String redisKey) {
		return redisTemplate.hasKey(redisKey);
	}
}
