package com.jianxun.gaudi.sys.service.impl;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.sys.dao.SysNumberAssignDAO;
import com.jianxun.gaudi.sys.model.SysNumberAssign;
import com.jianxun.gaudi.sys.service.RedisService;
import com.jianxun.gaudi.sys.service.SysNumberAssignService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

@Service
public class SysNumberAssignServiceImpl implements SysNumberAssignService {

    private final static String KEY_OF_ASSIGN_NUMBER = "NumberAssign";

    @Autowired
    RedisService redisSV;

    @Autowired
    SysNumberAssignDAO numDao;

    @Override
    public String assignNumber(String type) {
        String nextValue = "";
        String strJson = "";
        long nextNumber = 0;
        SysNumberAssign typeVO = null;
        Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss")
                .create();
        String keyOfCurrentValue = String.format("%s:CurrentValue", type);
        String nowtime = "";

        try {
            strJson = redisSV.hget(KEY_OF_ASSIGN_NUMBER, type);

            if(StringUtils.isNotBlank(strJson)) {
                typeVO = gson.fromJson(strJson, SysNumberAssign.class);
            }else {
                //get type, pre_fix, length from DB
                typeVO = numDao.selectByPrimaryKey(type);

                if(typeVO != null) {
                    //save in redis
                    redisSV.hset(KEY_OF_ASSIGN_NUMBER, type, gson.toJson(typeVO));
                }
            }

            if(typeVO == null){
                throw new MyException(ExceptionMsg.NUMBER_TYPE_WITHOUT.getCode(), ExceptionMsg.NUMBER_TYPE_WITHOUT.getMsg());
            }else {
                if("Y".equalsIgnoreCase(typeVO.getWithDate())) {
                    Date now = new Date();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                    nowtime = sdf.format(now);

                    String redisKey = String.format("%s:%s:%s", KEY_OF_ASSIGN_NUMBER,typeVO.getType(),nowtime);
                    nextNumber = redisSV.incr(redisKey,1);

                    if(nextNumber == 1) {
                        redisSV.expire(redisKey, 2, TimeUnit.DAYS); //2 days expire
                    }
                }else {
                    nextNumber = redisSV.hincrBy(KEY_OF_ASSIGN_NUMBER, keyOfCurrentValue, 1);

                    if(nextNumber == 0 || nextNumber == 1){
                        SysNumberAssign sysNumberAssign = numDao.selectByPrimaryKey(type);
                        Long maxNumber = numDao.selectSql(sysNumberAssign.getQuerySql());
                        nextNumber = maxNumber + 1;
                        redisSV.hset(KEY_OF_ASSIGN_NUMBER, keyOfCurrentValue,nextNumber + "");
                    }
                }

                nextValue = FormatNumber(type, nextNumber, typeVO, nowtime);
            }
        }catch(Exception ex) {
            throw ex;
        }
        return nextValue;
    }

    /**
     * format number
     * @param type
     * @param orderNumber
     * @param typeVO
     * @param nowtime
     * @return
     */
    private static String FormatNumber(String type, Long orderNumber, SysNumberAssign typeVO, String nowtime){
        String result;
        int iLength = typeVO.getLength();
        String pre_fix = StringUtils.isEmpty(typeVO.getPreFix()) ? "":typeVO.getPreFix();

        if(typeVO.getPreFix() != null){
            iLength = typeVO.getLength() - typeVO.getPreFix().length();
        }

        if("Y".equalsIgnoreCase(typeVO.getWithDate())) {
            result = String.format("%s%s%s", pre_fix, nowtime, StringUtils.leftPad(orderNumber.toString(), iLength, "0"));
        }else {
            result = String.format("%s%s", pre_fix, StringUtils.leftPad(orderNumber.toString(), iLength,"0"));
        }

        return result;
    }
}
