package com.jianxun.gaudi.sys.service.impl;


import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Maps;
import com.jianxun.gaudi.common.consts.SystemConsts;
import com.jianxun.gaudi.common.enums.ExceptionMsg;
import com.jianxun.gaudi.common.enums.RedisKeyPrefix;
import com.jianxun.gaudi.common.exception.MyException;
import com.jianxun.gaudi.common.vo.TokenVO;
import com.jianxun.gaudi.sys.dao.SysUserDAO;
import com.jianxun.gaudi.sys.model.SysUser;
import com.jianxun.gaudi.sys.model.SysUserExample;
import com.jianxun.gaudi.sys.service.RedisService;
import com.jianxun.gaudi.sys.service.SysUserService;
import com.jianxun.gaudi.sys.vo.SysUserVO;
import com.jianxun.gaudi.utils.BeanUtils;
import com.jianxun.gaudi.utils.DateHelper;
import com.jianxun.gaudi.utils.JwtUtil;


@Service
@Transactional
public class SysUserServiceImpl implements SysUserService{
	@Autowired
	SysUserDAO dao;
	
	@Resource
	private RedisService redisService;

	@Override
	public SysUser getUserById(String userId) {
		// TODO Auto-generated method stub
		return dao.selectByPrimaryKey(userId);
	}

	@Override
	public SysUserVO login(String account,String password) throws MyException {
		// TODO Auto-generated method stub
		SysUserExample example = new SysUserExample();
		example.createCriteria().andAccountEqualTo(account)
			.andPasswordEqualTo(password);
		List<SysUser> lstUser = dao.selectByExample(example);
		
		if(lstUser.size() == 0 || !lstUser.get(0).getEnable()) {
			//cannot find user or this user is disabled
			throw new MyException(ExceptionMsg.USER_NOT_EXIST.getCode(),ExceptionMsg.USER_NOT_EXIST.getMsg());
		}
		
		SysUserVO rspVo = new SysUserVO();
		BeanUtils.copyProperties(lstUser.get(0), rspVo);
		
		//create token
		TokenVO token = JwtUtil.createToken(rspVo.getUserId(),SystemConsts.RedisExpireTime.TOKEN_EXPIRE_TIME);
		rspVo.setToken(token.getToken());
			
		//get role list by userId
//		List<SysRole> roles = roleMapper.selectByUserId(rspVo.getUserId());
//		rspVo.setRoles(roles);
		
		//List<String> lstRoleId = roles.stream().map(p -> p.getRoleId()).distinct().collect(Collectors.toList());
		
		//create refresh token
		TokenVO refreshToken = JwtUtil.createToken(rspVo.getUserId(),SystemConsts.RedisExpireTime.REFRESH_TOKEN_EXPIRE_TIME);
		rspVo.setRefreshToken(refreshToken.getToken());
		
		//save token in redis
		String createTime = DateHelper.formatDateToStr(token.getCreateTime());
		Map<String,String> mapValue = Maps.newHashMap();
		mapValue.put("userId", rspVo.getUserId());
		mapValue.put("account", rspVo.getAccount());
		mapValue.put("userFullname", rspVo.getUserFullname());
		mapValue.put("token", rspVo.getToken());
		mapValue.put("refreshToken", rspVo.getRefreshToken());
		//mapValue.put("roles", StringUtils.join(lstRoleId, ','));
		mapValue.put("createTime", createTime);
		mapValue.put("updateTime", createTime);
		String roleId ="";
		if(!StringUtils.isEmpty(lstUser.get(0).getRoleId())) {
			roleId=lstUser.get(0).getRoleId();
		}
		mapValue.put("roleId", roleId);
		mapValue.put("expireTime", DateHelper.formatDateToStr(token.getExpireTime()));
		
		redisService.saveHash(mapValue, RedisKeyPrefix.TOKEN_CACHE, rspVo.getUserId(), SystemConsts.RedisExpireTime.REFRESH_TOKEN_EXPIRE_TIME);
		
		return rspVo;
	}
}
