package com.jianxun.gaudi.controller.base;

import com.jianxun.gaudi.common.NameDictionary;
import com.jianxun.gaudi.common.ResponseErrResult;
import com.jianxun.gaudi.common.exception.MyException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.text.SimpleDateFormat;
import java.util.Date;

@Controller
public class BaseController{

	private Logger logger = LoggerFactory.getLogger(BaseController.class);

	@Autowired
	protected HttpServletRequest request;

	@ResponseBody
	@ExceptionHandler
	public ResponseErrResult exceptionHandler(HttpServletRequest request, Exception exception) {
		HttpServletResponse response = getResponse();
		ResponseErrResult responseerrResult = new ResponseErrResult();
		response.setHeader("Content-type", "text/html;charset=UTF-8");// 让浏览器用utf8来解析返回的数据
		response.setCharacterEncoding("UTF-8");// 告诉servlet用UTF-8转码，而不是用默认的ISO8859
		Date now = new Date();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		logger.error("time:" + sdf.format(now) + " err info:" + exception.toString());
		if (exception instanceof MyException) {
			MyException myException = (MyException) exception;
			responseerrResult.setCode(myException.getCode());
			responseerrResult.setMsg(myException.getMessage());
			return responseerrResult;
		}
		//responseResult.setData(exception.toString());
		responseerrResult.setCode("-1");
		responseerrResult.setMsg("请求失败 !");
		return responseerrResult;
	}

	public HttpServletRequest getRequest() {
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
		return request;
	}

	public HttpServletResponse getResponse() {
		HttpServletResponse response = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getResponse();
		return response;
	}

	public HttpSession getSession() {
		HttpSession session = getRequest().getSession();
		return session;
	}

	public Long getLong(String key) {
		String value = request.getParameter(key);
		if (StringUtils.isNotBlank(value)) {
			return Long.parseLong(value.trim());
		}
		return null;
	}

	// ///////////////////////getHttpRequest()方法扩展//////////////////////////
	/**
	 * 根据参数名从HttpRequest中获取Double类型的参数值，无值则返回null .
	 * 
	 * @param key
	 *            .
	 * @return DoubleValue or null .
	 */
	public Double getDouble(String key) {
		String value = request.getParameter(key);
		if (StringUtils.isNotBlank(value)) {
			return Double.parseDouble(value.trim());
		}
		return null;
	}

	public Integer getInteger(String key) {
		String value = request.getParameter(key);
		if (StringUtils.isNotBlank(value)) {
			return Integer.parseInt(value.trim());
		}
		return null;
	}

	public String getString(String key) {
		String param = request.getParameter(key);
		if (StringUtils.isEmpty(param)) {
			return param;
		} else {
			return param.trim();
		}
	}

	protected int getPageNum() {// 当前页数
		String pageNumStr = request.getParameter("pageNumber");
		int pageNum = 1;
		if (StringUtils.isNotBlank(pageNumStr)) {
			pageNum = Integer.valueOf(pageNumStr);
		}
		return pageNum;
	}

	protected String getToken() {
		return request.getHeader(NameDictionary.AUTHORIZATION);
	}

	protected int getPageSize() {
		String numPerPageStr = request.getParameter("pageSize");
		int numPerPage = 20;
		if (StringUtils.isNotBlank(numPerPageStr)) {
			numPerPage = Integer.parseInt(numPerPageStr);
		}
		return numPerPage;
	}
}
