package com.jianxun.gaudi.controller.cp;

import com.google.gson.Gson;
import com.jianxun.gaudi.common.DataResult;
import com.jianxun.gaudi.common.ResponseResult;
import com.jianxun.gaudi.common.ResponseResultUtil;
import com.jianxun.gaudi.controller.base.BaseController;
import com.jianxun.gaudi.cp.model.CpComment;
import com.jianxun.gaudi.cp.vo.CpCommentFromVO;
import com.jianxun.gaudi.dubbo.consumer.cp.CpCommentDubboConsumerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/comment")
public class CpCommentController extends BaseController {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    CpCommentDubboConsumerService service;

    /**
     * 创建投诉表扬
     * @param token
     * @param cpComment
     * @return
     */
    @PostMapping("/createCpComment")
    public ResponseResult createCpComment(@RequestHeader("token") String token, @RequestBody CpComment cpComment) {
        logger.info("CpCommentController.createCpComment token:{},cpComment:{}",token, new Gson().toJson(cpComment));
        service.createCpComment(token, cpComment);
        return ResponseResultUtil.createSuccesResult();
    }

    /**
     * 获取投诉表扬Web
     * @param token
     * @param cpCommentFromVO
     * @return
     */
    @PostMapping("/getCpComment")
    public ResponseResult getCpComment(@RequestHeader("token") String token, @RequestBody CpCommentFromVO cpCommentFromVO) {
        logger.info("CpCommentController.getCpComment token:{},cpCommentFromVO:{}",token, new Gson().toJson(cpCommentFromVO));
        return ResponseResultUtil.createSuccesResult(new DataResult(service.getCpComment(token, cpCommentFromVO)));
    }

    /**
     * 获取投诉表扬列表App
     * @param token
     * @param cpCommentFromVO
     * @return
     */
    @PostMapping("/getCpCommentApp")
    public ResponseResult getCpCommentApp(@RequestHeader("token") String token, @RequestBody CpCommentFromVO cpCommentFromVO) {
        logger.info("CpCommentController.getCpCommentApp token:{},cpCommentFromVO:{}",token, new Gson().toJson(cpCommentFromVO));
        return ResponseResultUtil.createSuccesResult(new DataResult(service.getCpCommentApp(token, cpCommentFromVO)));
    }

    /**
     * 获取投诉表扬详细App
     * @param token
     * @param cpCommentFromVO
     * @return
     */
    @PostMapping("/getCpCommentDetailApp")
    public ResponseResult getCpCommentDetailApp(@RequestHeader("token") String token, @RequestBody CpCommentFromVO cpCommentFromVO) {
        logger.info("CpCommentController.getCpCommentDetailApp token:{},cpCommentFromVO:{}",token, new Gson().toJson(cpCommentFromVO));
        return ResponseResultUtil.createSuccesResult(service.getCpCommentDetailApp(token, cpCommentFromVO));
    }

    /**
     * 获取投诉表扬微信
     * @param token
     * @param cpCommentFromVO
     * @return
     */
    @PostMapping("/getCpCommentWX")
    public ResponseResult getCpCommentWX(@RequestHeader("token") String token, @RequestBody CpCommentFromVO cpCommentFromVO) {
        logger.info("CpCommentController.getCpCommentWX token:{},cpCommentFromVO:{}",token, new Gson().toJson(cpCommentFromVO));
        return ResponseResultUtil.createSuccesResult(new DataResult(service.getCpCommentWX(token, cpCommentFromVO)));
    }

    /**
     * 获取待回复数量
     * @param token
     * @return
     */
    @PostMapping("/getCommentAmount")
    public ResponseResult getCommentAmount(@RequestHeader("token") String token) {
        logger.info("CpCommentController.getCommentAmount token:{}", token);
        Integer amount = service.getCommentAmount(token);
        Map<String, Integer> map = new HashMap<>();
        map.put("commentAmount", amount);
        return ResponseResultUtil.createSuccesResult(map);
    }

}
