package com.jianxun.gaudi.controller.qs;

import cn.superdesk.libs.common.json.JsonUtils;
import com.alibaba.dubbo.common.json.JSONArray;
import com.github.pagehelper.PageInfo;
import com.jianxun.gaudi.common.ResponseResult;
import com.jianxun.gaudi.common.ResponseResultUtil;
import com.jianxun.gaudi.common.vo.StatisticParam;
import com.jianxun.gaudi.common.vo.StatisticRes;
import com.jianxun.gaudi.dubbo.consumer.cp.CpCommentDubboConsumerService;
import com.jianxun.gaudi.dubbo.consumer.qs.QuestionnaireSurveyDubboConsumerService;
import com.jianxun.gaudi.qs.param.*;
import com.jianxun.gaudi.qs.vo.*;
import com.jianxun.gaudi.utils.ExcelUtil;
import com.jianxun.gaudi.utils.HttpClientUtils;
import com.jianxun.gaudi.utils.HttpRequestUtil;
import com.jianxun.gaudi.utils.Pager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/Survey")
public class QuestionnaireSurveyController {
    private final Logger logger = LoggerFactory.getLogger(getClass());


    @Autowired
    QuestionnaireSurveyDubboConsumerService service;
    @Autowired
    CpCommentDubboConsumerService cpService;

    @Value("${file.templatePath}")
    private String templatePath;


    @Value("${file.uploadPath}")
    private String uploadPath;

    /**
     * 查询手动问卷
     *
     * @param
     * @param
     * @return
     * @throws Exception
     */
    @RequestMapping("/getQuestionnaire")
    @ResponseBody
    public ResponseResult<List<QuestionairesVO>> getQuestionnaire(@RequestBody QsUserParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.getQuestionnaire param:{}", JsonUtils.toJson(param));
        ResponseResult<List<QuestionairesVO>> result = new ResponseResult<List<QuestionairesVO>>();
        result.setData(service.getQuestionnaire(param));
        return result;
    }

    /**
     * 增加手动问卷
     *
     * @param
     * @param param
     * @return
     * @throws Exception
     */
    @RequestMapping("/addQuestionnaire")
    @ResponseBody
    public ResponseResult<Object> addQuestionnaire(@RequestBody QsQuestionnaireParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.addQuestionnaire param:{}", JsonUtils.toJson(param));
        service.addQuestionnaire(param);
        return ResponseResultUtil.createSuccesResult();
    }

    /**
     * 问卷发送详情
     *
     * @param
     * @param id
     * @return
     * @throws Exception
     */
    @RequestMapping("/questionnaireDetails/{id}")
    @ResponseBody
    public ResponseResult<QuestionnaireDetailsVO> questionnaireDetails(@PathVariable Integer id) throws Exception {
        logger.info("QuestionnaireSurveyController.questionnaireDetails id:{}", JsonUtils.toJson(id));
        ResponseResult<QuestionnaireDetailsVO> result = new ResponseResult<QuestionnaireDetailsVO>();
        result.setData(service.questionnaireDetails(id));
        return result;
    }

    /**
     * 问卷进度详情
     *
     * @param
     * @param param
     * @return
     * @throws Exception
     */
    @RequestMapping("/questionnaireProgress")
    @ResponseBody
    public ResponseResult<QuestionnaireProgressVO> questionnaireProgress(@RequestBody QuestionnaireProgressParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.getManager QuestionnaireProgressParam:{}", JsonUtils.toJson(param));

        QuestionnaireProgressVO questionnaireProgressVO = new QuestionnaireProgressVO();
        //已参与用户
        param.setType(1);
        List<QsQuestionnaireUserVO> questionnaireParticipated = service.questionnaireProgress(param);

        //未参与用户
        param.setType(0);
        List<QsQuestionnaireUserVO> questionnaireNotParticipated = service.questionnaireProgress(param);

        questionnaireProgressVO.setQuestionnaireParticipated(questionnaireParticipated);
        questionnaireProgressVO.setQuestionnaireNotParticipated(questionnaireNotParticipated);

        ResponseResult<QuestionnaireProgressVO> result = new ResponseResult<QuestionnaireProgressVO>();
        result.setData(questionnaireProgressVO);
        return result;
    }


    /**
     * 发送设置
     *
     * @param
     * @param param
     * @return
     * @throws Exception
     */
    @RequestMapping("/sendSettings")
    @ResponseBody
    public ResponseResult<Object> sendSettings(@RequestBody QsQuestionnaireSettingParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.sendSettings param:{}", JsonUtils.toJson(param));
        service.sendSettings(param);
        return ResponseResultUtil.createSuccesResult();
    }



    /**
     * 查询发送设置
     * @param
     * @param orgId
     * @return
     * @throws Exception
     */
    @RequestMapping("/queryQuestionnaireSettings/{orgId}")
    @ResponseBody

    public ResponseResult<QsQuestionnaireSettingVO> queryQuestionnaireSettings(@PathVariable Integer orgId) throws Exception {
        logger.info("QuestionnaireSurveyController.queryQuestionnaireSettings orgId:{}", JsonUtils.toJson(orgId));
        ResponseResult<QsQuestionnaireSettingVO> result = new ResponseResult<QsQuestionnaireSettingVO>();
        result.setData(service.queryQuestionnaireSettings(orgId));
        return result;
    }


    /**
     * 查询问卷管理
     *
     * @param
     * @param
     * @return
     * @throws Exception
     */
    @RequestMapping("/getQuestionnaireInformation")
    @ResponseBody
    public ResponseResult getQuestionnaireInformation(@ModelAttribute QuestionnaireInquiryParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.getQuestionnaireInformation param:{}", JsonUtils.toJson(param));
        if(null == param.getPageNum()){
            param.setPageNum(1);
        }
        if(null == param.getPageSize()){
            param.setPageSize(10);
        }
        ResponseResult<PageInfo<QsQuestionnaireVO>> result = new ResponseResult<PageInfo<QsQuestionnaireVO>>();
        result.setData(service.getQuestionnaireInformation(param));
        return result;
    }


    /**
     * 问卷催收
     *
     * @param
     * @param param
     * @return
     * @throws Exception
     */
    @RequestMapping("/questionnaireReminder")
    @ResponseBody
    public ResponseResult<Object> questionnaireReminder(@RequestBody QsQuestionnaireUrgeParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.questionnaireReminder param:{}", JsonUtils.toJson(param));
        service.questionnaireReminder(param);
        return ResponseResultUtil.createSuccesResult();
    }

    /**
     * 导出Excel
     *
     * @param
     * @param
     * @return
     * @throws Exception
     */
    @RequestMapping("/exportExcel")
    @ResponseBody
    public ResponseResult<Object> exportExcel( @RequestBody QuestionnaireProgressParam param) throws Exception {
        logger.info("QuestionnaireSurveyController.questionnaireReminder param:{}", JsonUtils.toJson(param));
        List<QsQuestionnaireUserVO> list = service.questionnaireProgress(param);

        Map map = new HashMap();
        map.put("data", list);
        ExcelUtil.exportExcel(templatePath, map,uploadPath);
        return ResponseResultUtil.createSuccesResult(uploadPath);
    }

    /**
     * 问卷调查统计数据
     * @param param
     * @return
     */
    @PostMapping("/statistics")
    public ResponseResult<List<StatisticRes>> getStatisticalData(@RequestBody StatisticParam param) {
        List<StatisticRes> res = new ArrayList<>();
        //问卷调查
        StatisticRes statisticRes = new StatisticRes();
        statisticRes.setType(4);
        statisticRes.setList(service.getStatisticalData(param));
        res.add(statisticRes);
        //投诉建议
        StatisticRes statisticRes1 = new StatisticRes();
        statisticRes1.setType(5);
        statisticRes1.setList(cpService.getStatisticalData(param));
        res.add(statisticRes1);
        return ResponseResultUtil.createSuccesResult(res);
    }

}
