package com.jianxun.gaudi.dubbo.consumer.qs;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.jianxun.gaudi.common.vo.StatisticParam;
import com.jianxun.gaudi.common.vo.StatisticSumVo;
import com.jianxun.gaudi.common.vo.StatisticVo;
import com.jianxun.gaudi.qs.param.*;
import com.jianxun.gaudi.qs.vo.*;
import com.jianxun.gaudi.utils.HttpRequestUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cglib.beans.BeanMap;
import org.springframework.stereotype.Component;

import com.alibaba.dubbo.config.annotation.Reference;
import com.jianxun.gaudi.qs.dubbo.DubboQuestionnaireSurveyService;
import com.jianxun.gaudi.qs.model.QsAnswer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Component
public class QuestionnaireSurveyDubboConsumerService {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    @Reference(version = "1.0.0", timeout = 10000000, retries = -1, interfaceClass = DubboQuestionnaireSurveyService.class)
    DubboQuestionnaireSurveyService qsService;

    @Value("${gaudi.qs.callback.urls}")
    String callBackUrls;


    /**
     * 获取问卷星管理页的链接
     *
     * @param token
     * @return
     * @throws Exception
     */
    public ManagerVO getManager(String token) throws Exception {
        return qsService.getManager(token);
    }

    /**
     * 问卷发送详情
     *
     * @param id
     * @return
     * @throws Exception
     */
    public QuestionnaireDetailsVO questionnaireDetails(Integer id) throws Exception {
        return qsService.questionnaireDetails(id);
    }

    /**
     * 问卷进度详情
     *
     * @param param
     * @return
     * @throws Exception
     */
    public  List<QsQuestionnaireUserVO> questionnaireProgress(QuestionnaireProgressParam param) throws Exception {
        return qsService.questionnaireProgress(param);
    }

    /**
     * 获取待参与问卷列表
     *
     * @param token
     * @return
     * @throws Exception
     */
    public List<QuestionairesVO> lstToFill(String token ) throws Exception {
        return qsService.lstToFill(token);
    }

    /**
     * 获取待参与问卷列表
     *
     * @param param
     * @return
     * @throws Exception
     */
    public List<QuestionairesVO> getQuestionnaire(QsUserParam param) throws Exception {
        return qsService.getQuestionnaire(param);
    }


    /**
     * 获取问卷发布列表
     *
     * @param param
     * @return
     * @throws Exception
     */
    public PageInfo<QsQuestionnaireVO> getQuestionnaireInformation(QuestionnaireInquiryParam param) throws Exception {
        return qsService.getQuestionnaireInformation(param);
    }
    /**
     * 获取已参与问卷列表
     *
     * @param token
     * @return
     * @throws Exception
     */
    public List<QuestionairesVO> lstFilled(String token) throws Exception {
        return qsService.lstFilled(token);
    }

    /**
     * 接收问卷星数据推送的回调函数
     *
     * @param qsAnswerVO
     * @throws Exception
     */
    public void callBack(QsAnswerVO qsAnswerVO) throws Exception {
        QsAnswer qsAnswer = new QsAnswer();
        BeanUtils.copyProperties(qsAnswerVO, qsAnswer);

        qsService.callBack(qsAnswer);

        //获取扩展字段，平台标识放在该字段里面，用来判断服务之窗 or 高校 0：服务之窗 1：高校
        if (StringUtils.isNotEmpty(qsAnswerVO.getExtf())) {
            String[] extStrings = qsAnswerVO.getExtf().split(";");
            if (extStrings != null && extStrings.length > 0 && StringUtils.isNotEmpty(callBackUrls)) {
                sendToOtherPlatrom(extStrings, qsAnswerVO);
            }
        }
    }

    /**
     * 发送回调数据至其他子平台
     *
     * @param extStrings
     * @param qsAnswerVO
     */
    private void sendToOtherPlatrom(String[] extStrings, QsAnswerVO qsAnswerVO) {
        int index = Integer.parseInt(extStrings[0]);
        String[] callBackUrlArray = callBackUrls.split(";");

        if (callBackUrlArray != null && callBackUrlArray.length > index) {
            String callBackUrl = callBackUrlArray[index];
            if (StringUtils.isEmpty(callBackUrl)) {
                return;
            }
            logger.info("HttpRequestUtil.send callBackUrl:" + callBackUrl + "----qsAnswerVO:" + new Gson().toJson(qsAnswerVO));
            String resultStr = HttpRequestUtil.send(callBackUrl, JSON.toJSONString(qsAnswerVO), "POST", "application/json");
            logger.info("HttpRequestUtil.send result:" + resultStr);
        }
    }

    /**
     * 获取问卷答案链接
     *
     * @param token
     * @param questionairesVO
     * @return
     * @throws Exception
     */
    public QsAnswerVO getAnswerUrl(String token, QuestionairesVO questionairesVO) throws Exception {
        return qsService.getAnswerUrl(token, questionairesVO);
    }

    public static <T> Map<String, String> beanToMap(T bean) {
        Map<String, String> map = Maps.newHashMap();
        if (bean != null) {
            BeanMap beanMap = BeanMap.create(bean);
            for (Object key : beanMap.keySet()) {
                map.put(key + "", beanMap.get(key) == null ? null : beanMap.get(key).toString());
            }
        }
        return map;
    }

    public void addQuestionnaire(QsQuestionnaireParam qsQuestionnaireParam) throws Exception {
        qsService.addQuestionnaire(qsQuestionnaireParam);
    }

    public void sendSettings(QsQuestionnaireSettingParam qsQuestionnaireParam) throws Exception {
        qsService.sendSettings(qsQuestionnaireParam);
    }
    public void modifyQuestionnaireUser(Integer userId)throws Exception{
        qsService.modifyQuestionnaireUser(userId);
    }
    public QsQuestionnaireSettingVO queryQuestionnaireSettings(Integer orgId)throws Exception{
        return qsService.queryQuestionnaireSettings(orgId);
    }
    public void questionnaireReminder(QsQuestionnaireUrgeParam param)throws Exception {
        qsService.questionnaireReminder(param);
    }

    public List<StatisticVo> getStatisticalData(StatisticParam param) {
        List<StatisticVo> statisticalData = qsService.getStatisticalData(param);

        return statisticalData;
    }
}
